unit fMain;
////////////////////////////////////////////////////////////////////////////////
// Unit:            fMain
//
// Application:     ICS POP3 Client (HADP99 Demo)
//
// Purpose:         Main form for POP client.
//
// Author:          Tres Seaver, Palladion Software.
//
// History:         1999/04/13      Original version
////////////////////////////////////////////////////////////////////////////////

interface

uses Windows
    , Messages
    , SysUtils
    , Classes
    , Graphics
    , Controls
    , Forms
    , Dialogs
    , StdCtrls
    , Pop3Prot
    , ExtCtrls
    , ComCtrls
    , uPopClient, SmtpProt
    ;

type

    TfrmICS_POP3_Demo = class( TForm )

      pnlTop                      : TPanel;
      Pop3Client                  : TPop3Cli;
    btnMailOptions: TButton;
      btnCheckMail                : TButton;
      lstHeaders                  : TListBox;
      splHeaderBody               : TSplitter;
      mmoBody                     : TMemo;
      pnlStatus                   : TStatusBar;
      rgpHeadersBody              : TRadioGroup;
    btnSendMail: TButton;
    btnDisconnect: TButton;

      procedure FormCreate( Sender : TObject );
      procedure FormDestroy( Sender : TObject );

      procedure ConfigureMailOptions( Sender : TObject );
      procedure CheckMail( Sender : TObject );

      procedure DisplayStatus( Sender : TObject
                             ; Msg    : String
                             );

      procedure BeginList( Sender : TObject );
      procedure AddMessage( Sender : TObject );
      procedure EndList( Sender : TObject );

      procedure POP3ConnectDone( Sender: TObject
                               ; RqType: TPop3Request
                               ; Error: Word
                               );

      procedure POP3ListDone( Sender    : TObject
                            ; RqType    : TPop3Request
                            ; Error     : Word
                            );

      procedure POP3TopDone( Sender     : TObject
                           ; RqType     : TPop3Request
                           ; Error      : Word
                           );

      procedure POP3SingleDone( Sender  : TObject
                              ; RqType  : TPop3Request
                              ; Error   : Word
                              );

      procedure FetchMsgBody(             Sender : TObject );
      procedure AddHeaderLine(            Sender : TObject );
      procedure BeginMessage(             Sender : TObject );
      procedure AddMessageLine(           Sender : TObject );
      procedure DumpMsgHeaders(           Sender : TObject );
      procedure FetchRequestedMessage(    Sender : TObject );
      procedure DeleteMessage(            Sender : TObject );
    procedure SendMail(Sender: TObject);
    procedure DisconnectFromPOP(Sender: TObject);

    private

      FHeaders      : TList;
      FStates       : POP3_FSM;


      function doConnect( opToMaybePend: TPop3NextProc ) : Boolean;

    end;

var
    frmICS_POP3_Demo: TfrmICS_POP3_Demo;

implementation

{$R *.DFM}

uses  fOptions
    , fSendMail
    ;

////////////////////////////////////////////////////////////////////////////////
// POP3_Functor -- Memento for next operation.
////////////////////////////////////////////////////////////////////////////////
type
    POP3_Functor = class
        FNextProc : TPop3NextProc;
        constructor Create( nextProc : TPop3NextProc );
        procedure Execute;
    end;


constructor POP3_Functor.Create( nextProc : TPop3NextProc );
begin
    FNextProc := nextProc;
end;


procedure POP3_Functor.Execute;
begin
    if Assigned( FNextProc ) then
        FNextProc();
end;

////////////////////////////////////////////////////////////////////////////////
// Standard form event handlers
////////////////////////////////////////////////////////////////////////////////

procedure TfrmICS_POP3_Demo.FormCreate( Sender : TObject );
begin
    FHeaders            := TList.Create;
    FStates             := POP3_FSM.Create( Pop3Client );
end;

procedure TfrmICS_POP3_Demo.FormDestroy( Sender : TObject );
begin
    FStates.Free;
    FHeaders.Free;
end;

procedure TfrmICS_POP3_Demo.ConfigureMailOptions( Sender : TObject );
begin
    dlgMailServerOptions.POP3Client := self.Pop3Client;
    dlgMailServerOptions.SMTPClient := frmSMTPMessage.SMTPClient;
    dlgMailServerOptions.ShowModal;
end;


procedure TfrmICS_POP3_Demo.CheckMail( Sender : TObject );
begin
    FStates.Push( POP3_Functor.Create( Nil ) );

    Pop3Client.MsgNum           := 1;
    Pop3Client.OnMessageBegin   := Nil;
    Pop3Client.OnMessageLine    := AddHeaderLine;
    Pop3Client.ONMessageEnd     := Nil;
    Pop3Client.OnRequestDone    := POP3TopDone;
    FStates.Push( POP3_Functor.Create( Pop3Client.Top ) );

    Pop3Client.MsgNum           := 0;
    Pop3Client.OnListBegin      := BeginList;
    Pop3Client.OnListEnd        := EndList;
    Pop3Client.OnListLine       := AddMessage;
    Pop3Client.OnRequestDone    := POP3ListDone;

    doConnect( Pop3Client.List );
end;


procedure TfrmICS_POP3_Demo.DisconnectFromPOP(Sender: TObject);
begin
    FStates.Push( POP3_Functor.Create( Nil ) );
    Pop3Client.OnRequestDone    := POP3ConnectDone;
    Pop3Client.Quit;
end;


procedure TfrmICS_POP3_Demo.SendMail( Sender : TObject );
begin
    frmSMTPMessage.Show;
end;


procedure TfrmICS_POP3_Demo.DisplayStatus( Sender   : TObject
                                         ; Msg      : String
                                         );
begin
    pnlStatus.Panels[ 0 ].Text := Msg;
end;


procedure TfrmICS_POP3_Demo.FetchRequestedMessage( Sender : TObject );
begin
    case rgpHeadersBody.ItemIndex of
        0 :
            begin
                DumpMsgHeaders( Sender );
            end;
        1 :
            begin
                FetchMsgBody( Sender );
            end;
        2 :
            begin
                DeleteMessage( Sender );
            end;
    end;
end;


////////////////////////////////////////////////////////////////////////////////
// POP3 State machine methods -- Connect FSM
////////////////////////////////////////////////////////////////////////////////

function TfrmICS_POP3_Demo.doConnect( opToMaybePend: TPop3NextProc ) : Boolean;
begin
    with Pop3Client do
    begin
        Result := not Connected;
        if Result then
        begin
            FStates.Push( POP3_Functor.Create( optoMaybePend ) );
            OnRequestDone   := POP3ConnectDone;
            Connect;
        end
        else
            opToMaybePend();
    end;
end;


procedure TfrmICS_POP3_Demo.POP3ConnectDone( Sender  : TObject
                                           ; RqType  : TPop3Request
                                           ; Error   : Word
                                           );
begin
    with Pop3Client do
    begin
        if Error <> 0 then
            MessageDlg( Pop3Client.ErrorMessage, mtError, [ mbOk ], 0 );
        case RqType of
            pop3Connect : User;
            pop3User    : Pass;
            pop3Pass,
            pop3Quit    : with POP3_Functor( FStates.Pop ) do
                          begin
                              Execute;
                              Free;
                          end;
        end;
    end;
end;


////////////////////////////////////////////////////////////////////////////////
// POP3 State machine methods -- LIST FSM
////////////////////////////////////////////////////////////////////////////////



procedure TfrmICS_POP3_Demo.BeginList( Sender : TObject );
begin
    FHeaders.Clear;
end;


procedure TfrmICS_POP3_Demo.AddMessage( Sender : TObject );
var
    msg     : POP3_Message;

begin
    msg             := POP3_Message.Create( Pop3Client );
    FHeaders.Add( msg );
end;


procedure TfrmICS_POP3_Demo.EndList( Sender : TObject );
begin
    lstHeaders.Clear;
end;

procedure TfrmICS_POP3_Demo.POP3ListDone( Sender  : TObject
                                        ; RqType  : TPop3Request
                                        ; Error   : Word
                                        );
begin
    if RqType <> pop3List then
        raise Exception.CreateFmt( 'Unexpected request completion: %d'
                                 , [ Ord( RqType ) ] );
    if Error <> 0 then
        raise Exception.CreateFmt( 'Error in LIST request: %d'
                                 , [ Error ] );
    with POP3_Functor( FStates.Pop ) do
    begin
        if FHeaders.Count > 0 then
            Execute;
        Free;
    end;
end;


////////////////////////////////////////////////////////////////////////////////
// POP3 State machine methods -- TOP FSM
////////////////////////////////////////////////////////////////////////////////


procedure TfrmICS_POP3_Demo.AddHeaderLine( Sender : TObject );
var
    msg     : POP3_Message;

begin
    msg                      := Pop3_Message( FHeaders[ Pop3Client.MsgNum - 1 ] );
    msg.ParseHeaderLine( Pop3Client.LastResponse );
end;


procedure TfrmICS_POP3_Demo.POP3TopDone( Sender  : TObject
                                          ; RqType  : TPop3Request
                                          ; Error   : Word
                                          );
var
    iMsg        : Integer;
    msg         : POP3_Message;

begin
    if RqType <> pop3Top then
        raise Exception.CreateFmt( 'Unexpected request completion: %d'
                                 , [ Ord( RqType ) ] );
    if Error <> 0 then
        raise Exception.CreateFmt( 'Error in TOP request: %d'
                                 , [ Error ] );

    with Pop3Client do
    begin
        if MsgNum >= FHeaders.Count then    // done
        begin
            for iMsg := 0 to FHeaders.Count - 1 do
            begin
                msg := POP3_Message( FHeaders[ iMsg ] );
                lstHeaders.Items.AddObject( Format( '%-60s %-80s [%8d]'
                                                  , [ msg.From
                                                    , msg.Subject
                                                    , msg.MsgSize
                                                    ] )
                                          , msg
                                          );
            end;
            FHeaders.Clear;
            with POP3_Functor( FStates.Pop ) do
            begin
                Execute;
                Free;
            end;
        end
        else
        begin
            MsgNum := MsgNum + 1;
            Top;
        end;
    end;
end;


function selectedMessage( lst : TListBox ) : POP3_Message;
var
    iMsg        : Integer;

begin
    Result  := Nil;
    iMsg    := 0;

    while not Assigned( Result ) and ( iMsg < lst.Items.Count ) do
    begin
        if lst.Selected[ iMsg ] then
            Result := POP3_Message( lst.Items.Objects[ iMsg ] )
        else
            Inc( iMsg );
    end;

    if not Assigned( Result ) then
        raise Exception.Create( 'Please select a header!' );
end;


procedure TfrmICS_POP3_Demo.FetchMsgBody( Sender : TObject );
var
    msg         : POP3_Message;

begin
    msg         := selectedMessage( lstHeaders );
    mmoBody.Lines.Clear;

    FStates.Push( POP3_Functor.Create( Nil ) );

    with Pop3Client do
    begin
        MsgLines        := 0;
        MsgNum          := msg.MsgNumber;
        OnMessageBegin  := BeginMessage;
        OnMessageLine   := AddMessageLine;
        OnMessageEnd    := Nil;
        OnRequestDone   := POP3SingleDone;
        Retr;
    end;
end;


procedure TfrmICS_POP3_Demo.DumpMsgHeaders( Sender : TObject );
var
    msg         : POP3_Message;

begin
    msg         := selectedMessage( lstHeaders );
    mmoBody.Lines.Assign( msg.AllHeaders );
end;


procedure TfrmICS_POP3_Demo.BeginMessage( Sender : TObject );
begin
    mmoBody.Lines.Clear;
end;


procedure TfrmICS_POP3_Demo.AddMessageLine( Sender : TObject );
begin
    mmoBody.Lines.Add( Pop3Client.LastResponse );
end;


procedure TfrmICS_POP3_Demo.DeleteMessage( Sender : TObject );
var
    msg         : POP3_Message;

begin
    msg         := selectedMessage( lstHeaders );
    mmoBody.Lines.Clear;

    FStates.Push( POP3_Functor.Create( Nil ) );

    with Pop3Client do
    begin
        MsgNum          := msg.MsgNumber;
        OnMessageBegin  := BeginMessage;
        OnMessageLine   := AddMessageLine;
        OnMessageEnd    := Nil;
        OnRequestDone   := POP3SingleDone;
        Dele;
    end;
end;


procedure TfrmICS_POP3_Demo.POP3SingleDone( Sender   : TObject
                                          ; RqType   : TPop3Request
                                          ; Error    : Word
                                          );
begin
    with POP3_Functor( FStates.Pop ) do
    begin
        Execute;
        Free;
    end;
end;

end.


